/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceHeader");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyLongElement");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyShortElement");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");

(function() {

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
var ServerPropertyServiceHeader = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceHeader;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var ServerPropertyLongElement = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyLongElement;
var ServerPropertyShortElement = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyShortElement;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var arrayContainsObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayContainsObject;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService", [dijit._Widget], {
	
	enableToggle: true,
	initialMode: "view",		// valid values are "view" or "edit"
	descriptionMode: "tooltip", // valid values are "tooltip" or "table"
	serviceCallsPending: 0,
	
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService.superclass.postCreate.apply(this);
		checkNotNull(this.serviceInfo, "this.serviceInfo");
		checkNotNull(this.displayIn, "this.displayIn");
		
		if (this.propertySortOrder) {
			this._propertySortHash = new Object();
			for (var i=0; i < this.propertySortOrder.length; i++) {
				this._propertySortHash[this.propertySortOrder[i]] = i;
			}
		}
		this._implementationClass = this.serviceInfo.implementationClassName;
		this._initialize();
	},
	
	// a function that can be connected to by implementing code in order to
	// take action when a response is received from the server
	_onServiceResponse: function () {},
	
	_initialize: function () {
		this._header = new ServerPropertyServiceHeader({
			"_enableToggle": this.enableToggle,
			"headerClass": this.serviceHeaderClass,
			"customTitle": this.customTitle == null ? this._implementationClass : this.customTitle
		});
		this._serviceRow = new ServerPropertyServiceRow({
			"_enableToggle": this.enableToggle,
			"rowClass": this.propertiesRowClass,
			"_descriptionMode": this.descriptionMode
		});
		this._header.domNode._defaultSortVal = this._implementationClass;
		this._serviceRow.domNode._defaultSortVal = this._implementationClass;
		if (this.enableToggle) {
			this.connect(this._serviceRow.domNode, "onclick", function (e) {
				this._toggleEdit(true, e.target);
			});
			this.connect(this._header._editLink, "onclick", function () {
				this._toggleEdit(true);
			});
			this.connect(this._header._viewLink, "onclick", function () {
				this._toggleEdit(false);
			});
		}
		this._properties = new Array();
		
		if (this.configInfo) {
			/*
			 * If this object has a parent, it should not need to make another request for the service configuration data.  This
			 * information should already be defined in the configInfo object
			 * 
			 * We use setTimeout here to yield, and allow the parent component to catch up before continuing to create the ServerPropertyElements.
			 */
			setTimeout(dojo.hitch(this, function () {
				this._handleServiceProps(this.configInfo);
			}), 0);
		}
		else {
			this.serviceCallsPending++;
			var srh= new ServiceResponseHandler(this, "_handleServiceProps", "_handleServiceFetchError");
			ServerStatusClient.getServiceConfiguration(srh, this._implementationClass);
		}
	},
	
	_handleServiceProps: function(serviceInfo) {
		if (serviceInfo.properties != null && serviceInfo.properties.length > 0) {
			var longExists = false;
			var shortExists = false;
			for (var i=0; i < serviceInfo.properties.length; i++) {
				var curElement;
				if (serviceInfo.properties[i].propertyType == "STRING" && !this.forceShortFields) {
					longExists = true;
					curElement = new ServerPropertyLongElement({
						"elementInfo": serviceInfo.properties[i],
						"_prefix": this._implementationClass + ".",
						"_descriptionMode": this.descriptionMode,
						"displayIn": this._serviceRow._longPropertiesTableBody,
						"customSortValue": this._propertySortHash ? this._propertySortHash[serviceInfo.properties[i].name] : null,
						"implementationClass": this._implementationClass
					});
					this.connect(curElement, "_hide", "_checkHideLongHeaders");
					this.connect(curElement, "_show", "_showLongHeaders");
				}
				else {
					shortExists = true;
					curElement = new ServerPropertyShortElement({
						"elementInfo": serviceInfo.properties[i],
						"_prefix": this._implementationClass + ".",
						"_descriptionMode": this.descriptionMode,
						"displayIn": this._serviceRow._shortPropertiesTableBody,
						"customSortValue": this._propertySortHash ? this._propertySortHash[serviceInfo.properties[i].name] : null,
						"implementationClass": this._implementationClass
					});
					this.connect(curElement, "_hide", "_checkHideShortHeaders");
					this.connect(curElement, "_show", "_showShortHeaders");
				}
				this._properties.push(curElement);
				this.connect(curElement, "_show", "_show");
				this.connect(curElement, "_hide", "_checkHide");
				this.connect(curElement, "_markChanged", "_markChanged");
				this.connect(curElement, "_markUnchanged", "_checkUnchanged");
				this.connect(curElement, "_markInvalid", "_markInvalid");
				this.connect(curElement, "_markValid", "_checkValid");
				this.connect(curElement, "_checkValidity", "_fetchValidity");
				if (arrayContainsObject(this.hiddenPropertyList, serviceInfo.properties[i].name)) {
					curElement._hide();
					curElement._show = function () {}; // do nothing even when filters are cleared
				}
			}
			if (!longExists) {
				this._serviceRow._longPropertiesTableHeader.style.display = "none";
			}
			if (!shortExists) {
				this._serviceRow._shortPropertiesTableHeader.style.display = "none";
			}
			if (this.initialMode == "edit") {
				this._toggleEdit(true);
			}
			else {
				this._toggleEdit(false);
			}
			if (!this.configInfo) {
				this.serviceCallsPending--;
			}
			this._onServiceResponse();
			this._fetchValidity();
			this._show();
		}
		else {
			if (!this.configInfo) {
				this.serviceCallsPending--;
			}
			this._onServiceResponse();
		}
	},
	
	_handleServiceFetchError: function (errorObj) {
		// ignore service fetch errors since this may happen for uninitialized services
		this.serviceCallsPending--;
		this._onServiceResponse();
	},
	
	_show: function () {
		this._header.domNode.style.display = "";
		this._serviceRow.domNode.style.display = "";
		this._showing = true;
		
		if (!this._initialized) {
			var beforeNode = this.displayIn.firstChild;
			while (beforeNode != null && this._sortFunction(this._header.domNode, beforeNode) > 0) {
				beforeNode = beforeNode.nextSibling;
			}
			if (beforeNode == null) {
				this.displayIn.appendChild(this._header.domNode);
				this.displayIn.appendChild(this._serviceRow.domNode);
			}
			else {
				this.displayIn.insertBefore(this._header.domNode, beforeNode);
				this.displayIn.insertBefore(this._serviceRow.domNode, beforeNode);
			}
			this._initialized = true;
		}
	},
	
	_sortFunction: function (a, b) {
		if (a._customSortVal != null || b._customSortVal != null) {
			if (a._customSortVal != null && b._customSortVal != null) {
				if (a._customSortVal < b._customSortVal) {
					return -1;
				}
				else if (a._customSortVal > b._customSortVal) {
					return 1;
				}
				else {
					return 0;
				}
			}
			else if (a._customSortVal == null) {
				return 1;
			}
			else {
				return -1;
			}
		}
		else {
			if (a._defaultSortVal < b._defaultSortVal) {
				return -1;
			}
			else if (a._defaultSortVal > b._defaultSortVal) {
				return 1;
			}
			else {
				return 0;
			}
		}
	},
	
	_checkHide: function () {
		var hide = true;
		for (var i=0; i < this._properties.length; i++) {
			if (this._properties[i]._showing) {
				hide = false;
				break;
			}
		}
		if (hide) {
			this._hide();
		}
	},
	
	_checkHideLongHeaders: function () {
		var hide = true;
		for (var i=0; i < this._properties.length; i++) {
			if (this._properties[i].propertyType == "STRING" && this._properties[i]._showing) {
				hide = false;
				break;
			}
		}
		if (hide) {
			this._serviceRow._longPropertiesTableHeader.style.display = "none";
		}
	},
	
	_checkHideShortHeaders: function () {
		var hide = true;
		for (var i=0; i < this._properties.length; i++) {
			if (this._properties[i].propertyType != "STRING" && this._properties[i]._showing) {
				hide = false;
				break;
			}
		}
		if (hide) {
			this._serviceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
	
	_showLongHeaders: function () {
		this._serviceRow._longPropertiesTableHeader.style.display = "";
	},
	
	_showShortHeaders: function () {
		this._serviceRow._shortPropertiesTableHeader.style.display = "";
	},
	
	_hide: function () {
		this._header.domNode.style.display = "none";
		this._serviceRow.domNode.style.display = "none";
		this._showing = false;
	},
	
	_markChanged: function () {
		this._changed = true;
	},
	
	_checkUnchanged: function () {
		change = false;
		for (var i=0; i < this._properties.length; i++) {
			if (this._properties[i]._changed) {
				change = true;
				break;
			}
		}
		this._changed = change;
	},
	
	_markInvalid: function () {
		this._invalid = true;
	},
	
	_checkValid: function () {
		invalid = false;
		for (var i=0; i < this._properties.length; i++) {
			if (this._properties[i]._invalid) {
				invalid = true;
				break;
			}
		}
		this._invalid = invalid;
	},
		
	_fetchValidity: function () {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: true
		};
		var srh = new ServiceResponseHandler(this, "_checkValidityCallback", "_checkValidityError");
		ServerStatusClient.postConfigurationValidation(srh, changeArgs);
	},
	
	_getAllValuesExceptSensitiveReplacements: function () {
		//don't send sensitive unless it's changed
		var changes = new Array();
		for (var i = 0; i < this._properties.length; i++) {
			if (this._properties[i].elementInfo.accessPolicy != "SENSITIVE" || this._properties[i]._changed) {
					changes.push(this._properties[i]._getChange());
			}
		}
		return changes;
	},

	_checkValidityCallback: function (msg) {
		for (var i=0; i < this._properties.length; i++) {
			this._properties[i]._markValid();
		}
		if (msg != null) {
			// sort by reverse alpha (warning, ok, error) to apply errors last
			if (msg.length > 1) {
				msg.sort(
					function(a, b) {
						return a.code < b.code ? 1 : -1;
					});
			}
			// loop through all the messages		
			for (var i = 0; i < msg.length; i++) {			
				for (var j = 0; j < this._properties.length; j++) {
					// apply message to the correct property name
					if (msg[i].name === this._properties[j]._name) {
						// send whole msg object so element can display according to type 
						this._properties[j]._markInvalid(msg[i]);
					}
				}
			}
		}
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
	
	_checkValidityError: function () {
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i]._validationPending = false;
		}
	},
	
	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		for (var i = 0; i < this._properties.length; i++) {
			if (this._properties[i]._changed) {
				changes.push(this._properties[i]._getChange());
				properties.push(this._properties[i]);
			}
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
	
	_toggleEdit: function (editMode, target) {
		if (editMode) {
			this._serviceRow.domNode.title = "";
			dojo.toggleClass(this._serviceRow._propertiesTable, "viewProperties", false);
			dojo.toggleClass(this._serviceRow._propertiesTable, "editProperties", true);
			dojo.toggleClass(this._header._editLink, "disabled", true);
			dojo.toggleClass(this._header._viewLink, "disabled", false);
			while (target != null && target.tagName != "TR") {
				target = target.parentNode;
			}
			if (target == null) {
				if (this._serviceRow._longPropertiesTableBody.rows.length > 0) {
					target = this._serviceRow._longPropertiesTableBody.rows[0];
				}
				else if (this._serviceRow._shortPropertiesTableBody.rows.length > 0) {
					target = this._serviceRow._shortPropertiesTableBody.rows[0];
				}
			}
			
			// the properties may not be fetched yet
			if (this._properties != null) {
				for (var i = 0; i < this._properties.length; i++) {
					// IE bug - won't display the fields unless the CSS changes
					if (dojo.isIE && this._properties[i]._propertyValue.parentNode != null) {
						this._properties[i]._propertyValue.parentNode.style.zoom = "normal";
						this._properties[i]._propertyValue.parentNode.style.zoom = 1;
					}
					if (this._properties[i]._propertyValue.parentNode != null && this._properties[i]._propertyValue.parentNode.parentNode == target) {
						this._properties[i]._editValueNode.focus();
					}
				}
			}
		}
		else {
			if (this.enableEdits) {
				this._serviceRow.domNode.title = "Click to edit";
			}
			else {
				this._serviceRow.domNode.title = "";
			}
			dojo.toggleClass(this._serviceRow._propertiesTable, "viewProperties", true);
			dojo.toggleClass(this._serviceRow._propertiesTable, "editProperties", false);
			dojo.toggleClass(this._header._editLink, "disabled", false);
			dojo.toggleClass(this._header._viewLink, "disabled", true);
		}
	}, 
	
	// public methods
	revertDefault: function () {
		//call revert and set validate pending for each property
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i].revertDefault();
			this._properties[i]._validationPending = true;
		}
		// then call validate to send all at once
		this._fetchValidity(); 
		this._checkUnchanged();
	},
	
	restoreSaved: function () {
		//call revert and set validate pending for each property
		for (var i = 0; i < this._properties.length; i++) {
			this._properties[i].restoreSaved();
			this._properties[i]._validationPending = true;
		}
		// then call validate to send all at once
		this._fetchValidity(); 
		this._checkUnchanged();
	},
	
	// adding validation test for robust path to service layer
	// consider moving this to config utility later
	testRobustValidity: function (handler) {
		var pendingUpdates = this._getAllValuesExceptSensitiveReplacements();
		if (pendingUpdates.length === 0) {
			return;
		}
		var changeArgs = {
			updates: dojo.toJson(pendingUpdates), 
			fast: false
		};
		ServerStatusClient.postConfigurationValidation(handler, changeArgs);		
	},

	hasOverrideValues: function () {
		for (var i = 0; i < this._properties.length; i++) {
			if (this._properties[i].hasOverride()){
				return true;
			}
		}
		return false;
	},
	
	getEditedValueForProperty: function(propertyName){
		return this.getServerPropertyElement(propertyName)._editValueNode.value;		
	},	
	
	getServerPropertyElement: function(propertyName){
		var index= arrayIndexWithPropertyOfObject(this._properties, "_name", propertyName);
		return this._properties[index];
	},
	
	getPendingServiceCalls: function() {
		return this.serviceCallsPending;
	}

});

})();